# generated by datamodel-codegen:
#   filename:  open_path.json

from __future__ import annotations

from pydantic import BaseModel


class OpenPath(BaseModel):
    """
    Open Path
    """

    path: str
