# generated by datamodel-codegen:
#   filename:  sensors.json

from __future__ import annotations

from pydantic import BaseModel, Extra, Field


class Sensors(BaseModel):
    """
    Sensors
    """

    class Config:
        extra = Extra.allow

    last_updated: dict[str, float] = Field(..., description="Last updated")
