# generated by datamodel-codegen:
#   filename:  response.json

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel, Extra, Field


class Response(BaseModel):
    """
    Response
    """

    class Config:
        extra = Extra.allow

    type: Optional[str] = Field(None, description="Type")
    message: Optional[str] = Field(None, description="Message")
