# generated by datamodel-codegen:
#   filename:  keyboard_key.json

from __future__ import annotations

from pydantic import BaseModel


class KeyboardKey(BaseModel):
    """
    Keyboard Key
    """

    key: str
