# generated by datamodel-codegen:
#   filename:  notification.json

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel


class Notification(BaseModel):
    """
    Keyboard
    """

    title: Optional[str] = None
    message: str
    icon: Optional[str] = None
