# generated by datamodel-codegen:
#   filename:  display.json

from __future__ import annotations

from pydantic import BaseModel, Extra, Field


class LastUpdated(BaseModel):
    """
    Last updated
    """

    class Config:
        extra = Extra.allow

    displays: float


class Display(BaseModel):
    """
    Display
    """

    class Config:
        extra = Extra.allow

    displays: list
    last_updated: LastUpdated = Field(..., description="Last updated")
